@@ -31,7 +31,6 @@
 
 #include <libempathy/empathy-time.h>
 #include <libempathy/empathy-utils.h>
-#include <libmissioncontrol/mc-profile.h>
 
 #include "empathy-theme-adium.h"
 #include "empathy-smiley-manager.h"
@@ -299,7 +298,46 @@ theme_adium_parse_body (EmpathyThemeAdium *theme,
 			       EMPATHY_PREFS_CHAT_SHOW_SMILEYS,
 			       &use_smileys);
 
-	if (use_smileys) {
+	/* Add <a href></a> arround links */
+	uri_regex = empathy_uri_regex_dup_singleton ();
+	match = g_regex_match (uri_regex, text, 0, &match_info);
+	if (match) {
+		gint last = 0;
+		gint s = 0, e = 0;
+
+		string = g_string_sized_new (strlen (text));
+		do {
+			gchar *real_url;
+
+			g_match_info_fetch_pos (match_info, 0, &s, &e);
+
+			if (s > last) {
+				/* Append the text between last link (or the
+				 * start of the message) and this link */
+				g_string_append_len (string, text + last, s - last);
+			}
+
+			/* Append the link inside <a href=""></a> tag */
+			real_url = empathy_make_absolute_url (text + s);
+
+			g_string_append (string, "<a href=\"");
+			g_string_append (string, real_url);
+			g_string_append (string, "\">");
+			g_string_append_len (string, text + s, e - s);
+			g_string_append (string, "</a>");
+
+			g_free (real_url);
+			last = e;
+		} while (g_match_info_next (match_info, NULL));
+
+		if (e < strlen (text)) {
+			/* Append the text after the last link */
+			g_string_append_len (string, text + e, strlen (text) - e);
+		}
+
+		g_free (ret);
+		text = ret = g_string_free (string, FALSE);
+	} else if (use_smileys) {
 		/* Replace smileys by a <img/> tag */
 		string = g_string_sized_new (strlen (text));
 		smileys = empathy_smiley_manager_parse (priv->smiley_manager, text);
@@ -326,41 +364,6 @@ theme_adium_parse_body (EmpathyThemeAdium *theme,
 		text = ret = g_string_free (string, FALSE);
 	}
 
-	/* Add <a href></a> arround links */
-	uri_regex = empathy_uri_regex_dup_singleton ();
-	match = g_regex_match (uri_regex, text, 0, &match_info);
-	if (match) {
-		gint last = 0;
-		gint s = 0, e = 0;
-
-		string = g_string_sized_new (strlen (text));
-		do {
-			g_match_info_fetch_pos (match_info, 0, &s, &e);
-
-			if (s > last) {
-				/* Append the text between last link (or the
-				 * start of the message) and this link */
-				g_string_append_len (string, text + last, s - last);
-			}
-
-			/* Append the link inside <a href=""></a> tag */
-			g_string_append (string, "<a href=\"");
-			g_string_append_len (string, text + s, e - s);
-			g_string_append (string, "\">");
-			g_string_append_len (string, text + s, e - s);
-			g_string_append (string, "</a>");
-
-			last = e;
-		} while (g_match_info_next (match_info, NULL));
-
-		if (e < strlen (text)) {
-			/* Append the text after the last link */
-			g_string_append_len (string, text + e, strlen (text) - e);
-		}
-
-		g_free (ret);
-		text = ret = g_string_free (string, FALSE);
-	}
 	g_match_info_free (match_info);
 	g_regex_unref (uri_regex);
 
@@ -475,7 +478,6 @@ theme_adium_append_html (EmpathyThemeAdium *theme,
 		} else if (theme_adium_match (&cur, "%time")) {
 			gchar *format = NULL;
 			gchar *end;
-
 			/* Time can be in 2 formats:
 			 * %time% or %time{strftime format}%
 			 * Extract the time format if provided. */
@@ -520,7 +522,6 @@ theme_adium_append_message (EmpathyChatView *view,
 	EmpathyThemeAdiumPriv *priv = GET_PRIV (theme);
 	EmpathyContact        *sender;
 	EmpathyAccount        *account;
-	McProfile             *account_profile;
 	gchar                 *dup_body = NULL;
 	const gchar           *body;
 	const gchar           *name;
@@ -544,8 +545,8 @@ theme_adium_append_message (EmpathyChatView *view,
 	/* Get information */
 	sender = empathy_message_get_sender (msg);
 	account = empathy_contact_get_account (sender);
-	account_profile = empathy_account_get_profile (account);
-	service_name = mc_profile_get_display_name (account_profile);
+	service_name = empathy_protocol_name_to_display_name
+		(empathy_account_get_protocol (account));
 	timestamp = empathy_message_get_timestamp (msg);
 	body = empathy_message_get_body (msg);
 	dup_body = theme_adium_parse_body (theme, body);
