@@ -35,7 +35,6 @@
 #include <libnotify/notification.h>
 
 #include <telepathy-glib/util.h>
-#include <libmissioncontrol/mission-control.h>
 
 #include <libempathy/empathy-contact.h>
 #include <libempathy/empathy-message.h>
@@ -50,6 +49,7 @@
 #include <libempathy-gtk/empathy-log-window.h>
 #include <libempathy-gtk/empathy-geometry.h>
 #include <libempathy-gtk/empathy-smiley-manager.h>
+#include <libempathy-gtk/empathy-sound.h>
 #include <libempathy-gtk/empathy-ui-utils.h>
 
 #include "empathy-chat-window.h"
@@ -77,6 +77,7 @@ typedef struct {
 	GtkUIManager *ui_manager;
 	GtkAction   *menu_conv_insert_smiley;
 	GtkAction   *menu_conv_favorite;
+	GtkAction   *menu_conv_toggle_contacts;
 
 	GtkAction   *menu_edit_cut;
 	GtkAction   *menu_edit_copy;
@@ -174,7 +175,7 @@ chat_tab_style_set_cb (GtkWidget *hbox,
 		"chat-window-tab-close-button");
 	context = gtk_widget_get_pango_context (hbox);
 
-	metrics = pango_context_get_metrics (context, hbox->style->font_desc,
+	metrics = pango_context_get_metrics (context, gtk_widget_get_style (hbox)->font_desc,
 		pango_context_get_language (context));
 	char_width = pango_font_metrics_get_approximate_char_width (metrics);
 	pango_font_metrics_unref (metrics);
@@ -406,7 +407,7 @@ chat_window_update_chat_tab (EmpathyChat *chat)
 	EmpathyContact        *remote_contact;
 	const gchar           *name;
 	const gchar           *id;
-	McAccount             *account;
+	EmpathyAccount        *account;
 	const gchar           *subject;
 	const gchar           *status = NULL;
 	GtkWidget             *widget;
@@ -427,7 +428,7 @@ chat_window_update_chat_tab (EmpathyChat *chat)
 	remote_contact = empathy_chat_get_remote_contact (chat);
 
 	DEBUG ("Updating chat tab, name=%s, account=%s, subject=%s, remote_contact=%p",
-		name, mc_account_get_unique_name (account), subject, remote_contact);
+		name, empathy_account_get_unique_name (account), subject, remote_contact);
 
 	/* Update tab image */
 	if (g_list_find (priv->chats_new_msg, chat)) {
@@ -459,7 +460,7 @@ chat_window_update_chat_tab (EmpathyChat *chat)
 	append_markup_printf (tooltip,
 			      "<b>%s</b><small> (%s)</small>",
 			      id,
-			      mc_account_get_display_name (account));
+			      empathy_account_get_display_name (account));
 
 	if (!EMP_STR_EMPTY (status)) {
 		append_markup_printf (tooltip, "\n<i>%s</i>", status);
@@ -547,24 +548,45 @@ chat_window_conv_activate_cb (GtkAction         *action,
 {
 	EmpathyChatWindowPriv *priv = GET_PRIV (window);
 	gboolean               is_room;
+	gboolean               active;
+	EmpathyContact        *remote_contact = NULL;
 
 	/* Favorite room menu */
 	is_room = empathy_chat_is_room (priv->current_chat);
 	if (is_room) {
 		const gchar *room;
-		McAccount   *account;
-		gboolean     found;
+		EmpathyAccount   *account;
+		gboolean     found = FALSE;
+		EmpathyChatroom *chatroom;
 
 		room = empathy_chat_get_id (priv->current_chat);
 		account = empathy_chat_get_account (priv->current_chat);
-		found = empathy_chatroom_manager_find (priv->chatroom_manager,
-						       account, room) != NULL;
+		chatroom = empathy_chatroom_manager_find (priv->chatroom_manager,
+						       account, room);
+		if (chatroom != NULL)
+			found = empathy_chatroom_is_favorite (chatroom);
 
 		DEBUG ("This room %s favorite", found ? "is" : "is not");
 		gtk_toggle_action_set_active (
 			GTK_TOGGLE_ACTION (priv->menu_conv_favorite), found);
 	}
 	gtk_action_set_visible (priv->menu_conv_favorite, is_room);
+
+	/* Show contacts menu */
+	g_object_get (priv->current_chat,
+		      "remote-contact", &remote_contact,
+		      "show-contacts", &active,
+		      NULL);
+	if (remote_contact == NULL) {
+		gtk_toggle_action_set_active (
+			GTK_TOGGLE_ACTION (priv->menu_conv_toggle_contacts),
+					   active);
+	}
+	gtk_action_set_visible (priv->menu_conv_toggle_contacts,
+				(remote_contact == NULL));
+	if (remote_contact != NULL) {
+		g_object_unref (remote_contact);
+	}
 }
 
 static void
@@ -582,7 +604,7 @@ chat_window_favorite_toggled_cb (GtkToggleAction   *toggle_action,
 {
 	EmpathyChatWindowPriv *priv = GET_PRIV (window);
 	gboolean               active;
-	McAccount             *account;
+	EmpathyAccount        *account;
 	const gchar           *room;
 	EmpathyChatroom       *chatroom;
 
@@ -593,19 +615,28 @@ chat_window_favorite_toggled_cb (GtkToggleAction   *toggle_action,
 	chatroom = empathy_chatroom_manager_find (priv->chatroom_manager,
 						  account, room);
 
-	if (active && !chatroom) {
+	if (chatroom == NULL) {
 		const gchar *name;
 
 		name = empathy_chat_get_name (priv->current_chat);
 		chatroom = empathy_chatroom_new_full (account, room, name, FALSE);
 		empathy_chatroom_manager_add (priv->chatroom_manager, chatroom);
 		g_object_unref (chatroom);
-		return;
-	}
-	
-	if (!active && chatroom) {
-		empathy_chatroom_manager_remove (priv->chatroom_manager, chatroom);
-	}
+  }
+
+	empathy_chatroom_set_favorite (chatroom, active);
+}
+
+static void
+chat_window_contacts_toggled_cb (GtkToggleAction   *toggle_action,
+				 EmpathyChatWindow *window)
+{
+	EmpathyChatWindowPriv *priv = GET_PRIV (window);
+	gboolean               active;
+
+	active = gtk_toggle_action_get_active (toggle_action);
+
+	empathy_chat_set_show_contacts (priv->current_chat, active);
 }
 
 static const gchar *
@@ -819,7 +850,7 @@ chat_window_help_contents_activate_cb (GtkAction         *action,
 {
 	EmpathyChatWindowPriv *priv = GET_PRIV (window);
 
-	empathy_url_show (priv->dialog, "ghelp:empathy?chat");
+	empathy_url_show (priv->dialog, "ghelp:empathy");
 }
 
 static void
@@ -910,10 +941,10 @@ chat_window_show_or_update_notification (EmpathyChatWindow *window,
 					 EmpathyChat *chat)
 {
 	EmpathyContact *sender;
-	char *header, *escaped;
+	const gchar *header;
+	char *escaped;
 	const char *body;
 	GdkPixbuf *pixbuf;
-	NotificationData *cb_data;
 	EmpathyChatWindowPriv *priv = GET_PRIV (window);
 	gboolean res;
 
@@ -927,38 +958,57 @@ chat_window_show_or_update_notification (EmpathyChatWindow *window,
 		}
 	}
 
-	cb_data = g_slice_new0 (NotificationData);
-	cb_data->chat = g_object_ref (chat);
-	cb_data->window = window;
-
 	sender = empathy_message_get_sender (message);
-	header = g_strdup_printf (_("New message from %s"),
-				  empathy_contact_get_name (sender));
+	header = empathy_contact_get_name (sender);
 	body = empathy_message_get_body (message);
 	escaped = g_markup_escape_text (body, -1);
 
-	pixbuf = empathy_misc_get_pixbuf_for_notification (sender, EMPATHY_IMAGE_NEW_MESSAGE);
-
 	if (priv->notification != NULL) {
 		notify_notification_update (priv->notification,
 					    header, escaped, NULL);
-		notify_notification_set_icon_from_pixbuf (priv->notification, pixbuf);
 	} else {
+		NotificationData *cb_data = cb_data = g_slice_new0 (NotificationData);
+
+		cb_data->chat = g_object_ref (chat);
+		cb_data->window = window;
+
 		priv->notification = notify_notification_new (header, escaped, NULL, NULL);
 		notify_notification_set_timeout (priv->notification, NOTIFY_EXPIRES_DEFAULT);
-		notify_notification_set_icon_from_pixbuf (priv->notification, pixbuf);
 
 		g_signal_connect (priv->notification, "closed",
 				  G_CALLBACK (chat_window_notification_closed_cb), cb_data);
 	}
 
+	pixbuf = empathy_misc_get_pixbuf_for_notification (sender, EMPATHY_IMAGE_NEW_MESSAGE);
+
+	if (pixbuf != NULL) {
+		notify_notification_set_icon_from_pixbuf (priv->notification, pixbuf);
+		g_object_unref (pixbuf);
+	}
+
 	notify_notification_show (priv->notification, NULL);
 
-	g_object_unref (pixbuf);
-	g_free (header);
 	g_free (escaped);
 }
 
+static void
+chat_window_set_highlight_room_tab_label (EmpathyChat *chat)
+{
+	gchar *markup;
+	GtkWidget *widget;
+
+	if (!empathy_chat_is_room (chat))
+		return;
+
+	markup = g_markup_printf_escaped (
+		"<span color=\"red\" weight=\"bold\">%s</span>",
+		empathy_chat_get_name (chat));
+
+	widget = g_object_get_data (G_OBJECT (chat), "chat-window-tab-label");
+	gtk_label_set_markup (GTK_LABEL (widget), markup);
+	g_free (markup);
+}
+
 static void
 chat_window_new_message_cb (EmpathyChat       *chat,
 			    EmpathyMessage    *message,
@@ -991,6 +1041,11 @@ chat_window_new_message_cb (EmpathyChat       *chat,
 		return;
 	}
 
+	if (!g_list_find (priv->chats_new_msg, chat)) {
+		priv->chats_new_msg = g_list_prepend (priv->chats_new_msg, chat);
+		chat_window_update_chat_tab (chat);
+	}
+
 	/* If empathy_chat_is_room () returns TRUE, that means it's a named MUC.
 	 * If empathy_chat_get_remote_contact () returns NULL, that means it's
 	 * an unamed MUC (msn-like).
@@ -1004,18 +1059,15 @@ chat_window_new_message_cb (EmpathyChat       *chat,
 	}
 
 	if (needs_urgency) {
-		if (!has_focus)
+		if (!has_focus) {
 			chat_window_set_urgency_hint (window, TRUE);
+			chat_window_set_highlight_room_tab_label (chat);
+		}
 
 		empathy_sound_play (GTK_WIDGET (priv->dialog),
 		    EMPATHY_SOUND_MESSAGE_INCOMING);
 		chat_window_show_or_update_notification (window, message, chat);
 	}
-
-	if (!g_list_find (priv->chats_new_msg, chat)) {
-		priv->chats_new_msg = g_list_prepend (priv->chats_new_msg, chat);
-		chat_window_update_chat_tab (chat);
-	}
 }
 
 static GtkNotebook *
@@ -1180,7 +1232,7 @@ chat_window_focus_in_event_cb (GtkWidget        *widget,
 	priv->chats_new_msg = g_list_remove (priv->chats_new_msg, priv->current_chat);
 
 	chat_window_set_urgency_hint (window, FALSE);
-	
+
 	/* Update the title, since we now mark all unread messages as read. */
 	chat_window_update_chat_tab (priv->current_chat);
 
@@ -1200,55 +1252,52 @@ chat_window_drag_data_received (GtkWidget        *widget,
 	if (info == DND_DRAG_TYPE_CONTACT_ID) {
 		EmpathyChat           *chat;
 		EmpathyChatWindow     *old_window;
-		McAccount             *account;
+		EmpathyAccount        *account;
+		EmpathyAccountManager *account_manager;
 		const gchar           *id;
 		gchar                **strv;
 		const gchar           *account_id;
 		const gchar           *contact_id;
 
-		id = (const gchar*) selection->data;
+		id = (const gchar*) gtk_selection_data_get_data (selection);
+		account_manager = empathy_account_manager_dup_singleton ();
 
 		DEBUG ("DND contact from roster with id:'%s'", id);
-		
+
 		strv = g_strsplit (id, "/", 2);
 		account_id = strv[0];
 		contact_id = strv[1];
-		account = mc_account_lookup (account_id);
+		account = empathy_account_manager_get_account (account_manager, account_id);
 		chat = empathy_chat_window_find_chat (account, contact_id);
 
 		if (!chat) {
-			EmpathyAccountManager *account_manager;
 			TpConnection *connection;
 
-			account_manager = empathy_account_manager_dup_singleton ();
-			connection = empathy_account_manager_get_connection (
-				account_manager, account);
+			connection = empathy_account_get_connection (account);
 
 			if (connection) {
 				empathy_dispatcher_chat_with_contact_id (
 					connection, contact_id, NULL, NULL);
 			}
 
-			g_object_unref (account_manager);
-			g_object_unref (account);
 			g_strfreev (strv);
 			return;
 		}
-		g_object_unref (account);
+		g_object_unref (account_manager);
 		g_strfreev (strv);
 
-		old_window = chat_window_find_chat (chat);		
+		old_window = chat_window_find_chat (chat);
 		if (old_window) {
 			if (old_window == window) {
 				gtk_drag_finish (context, TRUE, FALSE, time);
 				return;
 			}
-			
+
 			empathy_chat_window_move_chat (old_window, window, chat);
 		} else {
 			empathy_chat_window_add_chat (window, chat);
 		}
-		
+
 		/* Added to take care of any outstanding chat events */
 		empathy_chat_window_present_chat (chat);
 
@@ -1265,7 +1314,7 @@ chat_window_drag_data_received (GtkWidget        *widget,
 
 		DEBUG ("DND tab");
 
-		chat = (void *) selection->data;
+		chat = (void *) gtk_selection_data_get_data (selection);
 		old_window = chat_window_find_chat (*chat);
 
 		if (old_window) {
@@ -1279,7 +1328,7 @@ chat_window_drag_data_received (GtkWidget        *widget,
 				gtk_drag_finish (context, TRUE, FALSE, time);
 				return;
 			}
-			
+
 			priv->dnd_same_window = FALSE;
 		}
 
@@ -1369,6 +1418,7 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 				       "ui_manager", &priv->ui_manager,
 				       "menu_conv_insert_smiley", &priv->menu_conv_insert_smiley,
 				       "menu_conv_favorite", &priv->menu_conv_favorite,
+				       "menu_conv_toggle_contacts", &priv->menu_conv_toggle_contacts,
 				       "menu_edit_cut", &priv->menu_edit_cut,
 				       "menu_edit_copy", &priv->menu_edit_copy,
 				       "menu_edit_paste", &priv->menu_edit_paste,
@@ -1385,6 +1435,7 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 			      "menu_conv", "activate", chat_window_conv_activate_cb,
 			      "menu_conv_clear", "activate", chat_window_clear_activate_cb,
 			      "menu_conv_favorite", "toggled", chat_window_favorite_toggled_cb,
+			      "menu_conv_toggle_contacts", "toggled", chat_window_contacts_toggled_cb,
 			      "menu_conv_close", "activate", chat_window_close_activate_cb,
 			      "menu_edit", "activate", chat_window_edit_activate_cb,
 			      "menu_edit_cut", "activate", chat_window_cut_activate_cb,
@@ -1566,14 +1617,14 @@ empathy_chat_window_add_chat (EmpathyChatWindow *window,
 	/* If this window has just been created, position it */
 	if (priv->chats == NULL) {
 		empathy_geometry_load (chat_get_window_id_for_geometry (chat), &x, &y, &w, &h);
-		
+
 		if (x >= 0 && y >= 0) {
 			/* Let the window manager position it if we don't have
 			 * good x, y coordinates.
 			 */
 			gtk_window_move (GTK_WINDOW (priv->dialog), x, y);
 		}
-		
+
 		if (w > 0 && h > 0) {
 			/* Use the defaults from the ui file if we don't have
 			 * good w, h geometry.
@@ -1703,12 +1754,11 @@ empathy_chat_window_has_focus (EmpathyChatWindow *window)
 }
 
 EmpathyChat *
-empathy_chat_window_find_chat (McAccount   *account,
+empathy_chat_window_find_chat (EmpathyAccount   *account,
 			       const gchar *id)
 {
 	GList *l;
 
-	g_return_val_if_fail (MC_IS_ACCOUNT (account), NULL);
 	g_return_val_if_fail (!EMP_STR_EMPTY (id), NULL);
 
 	for (l = chat_windows; l; l = l->next) {
@@ -1724,7 +1774,7 @@ empathy_chat_window_find_chat (McAccount   *account,
 
 			chat = ll->data;
 
-			if (empathy_account_equal (account, empathy_chat_get_account (chat)) &&
+			if (account == empathy_chat_get_account (chat) &&
 			    !tp_strdiff (id, empathy_chat_get_id (chat))) {
 				return chat;
 			}
