@@ -23,6 +23,8 @@
 
 #include <string.h>
 
+#include <glib.h>
+
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
@@ -136,28 +138,37 @@ status_icon_update_notification (EmpathyStatusIcon *icon)
 	}
 
 	if (priv->event) {
-		pixbuf = empathy_misc_get_pixbuf_for_notification (priv->event->contact,
-								   priv->event->icon_name);
+		gchar *message_esc = NULL;
+
+		if (priv->event->message != NULL)
+			message_esc = g_markup_escape_text (priv->event->message, -1);
 
 		if (priv->notification) {
 			notify_notification_update (priv->notification,
-						    priv->event->header, priv->event->message,
+						    priv->event->header, message_esc,
 						    NULL);
 		} else {
 			priv->notification = notify_notification_new_with_status_icon
-				(priv->event->header, priv->event->message, NULL, priv->icon);
+				(priv->event->header, message_esc, NULL, priv->icon);
 			notify_notification_set_timeout (priv->notification,
 							 NOTIFY_EXPIRES_DEFAULT);
 
 			g_signal_connect (priv->notification, "closed",
 					  G_CALLBACK (status_icon_notification_closed_cb), icon);
+		}
 
- 		}
-		notify_notification_set_icon_from_pixbuf (priv->notification,
+		pixbuf = empathy_misc_get_pixbuf_for_notification (priv->event->contact,
+								   priv->event->icon_name);
+
+		if (pixbuf != NULL) {
+			notify_notification_set_icon_from_pixbuf (priv->notification,
 							  pixbuf);
+			g_object_unref (pixbuf);
+		}
+
 		notify_notification_show (priv->notification, NULL);
 
-		g_object_unref (pixbuf);
+		g_free (message_esc);
 	} else {
 		notification_close_helper (priv);
 	}
@@ -171,12 +182,12 @@ status_icon_update_tooltip (EmpathyStatusIcon *icon)
 
 	if (priv->event) {
 		if (priv->event->message != NULL)
-				tooltip = g_strdup_printf ("<i>%s</i>\n%s",
-							   priv->event->header,
-							   priv->event->message);
+				tooltip = g_markup_printf_escaped ("<i>%s</i>\n%s",
+								   priv->event->header,
+								   priv->event->message);
 		else
-				tooltip = g_strdup_printf ("<i>%s</i>",
-							   priv->event->header);
+				tooltip = g_markup_printf_escaped ("<i>%s</i>",
+								   priv->event->header);
 		gtk_status_icon_set_tooltip_markup (priv->icon, tooltip);
 	} else {
 		tooltip = g_strdup (empathy_idle_get_status (priv->idle));
@@ -229,13 +240,14 @@ status_icon_event_added_cb (EmpathyEventManager *manager,
 	DEBUG ("New event %p", event);
 
 	priv->event = event;
-	priv->showing_event_icon = TRUE;
-
-	status_icon_update_icon (icon);
-	status_icon_update_tooltip (icon);
+	if (event->must_ack) {
+		priv->showing_event_icon = TRUE;
+		status_icon_update_icon (icon);
+		status_icon_update_tooltip (icon);
+	}
 	status_icon_update_notification (icon);
 
-	if (!priv->blink_timeout) {
+	if (!priv->blink_timeout && priv->showing_event_icon) {
 		priv->blink_timeout = g_timeout_add (BLINK_TIMEOUT,
 						     (GSourceFunc) status_icon_blink_timeout_cb,
 						     icon);
@@ -260,7 +272,7 @@ status_icon_event_removed_cb (EmpathyEventManager *manager,
 
 	/* update notification anyway, as it's safe and we might have been
 	 * changed presence in the meanwhile
-	 */	
+	 */
 	status_icon_update_notification (icon);
 
 	if (!priv->event && priv->blink_timeout) {
@@ -302,18 +314,7 @@ status_icon_set_visibility (EmpathyStatusIcon *icon,
 	if (!visible) {
 		empathy_window_iconify (priv->window, priv->icon);
 	} else {
-		GList *accounts;
-
 		empathy_window_present (GTK_WINDOW (priv->window), TRUE);
-
-		/* Show the accounts dialog if there is no enabled accounts */
-		accounts = mc_accounts_list_by_enabled (TRUE);
-		if (accounts) {
-			mc_accounts_list_free (accounts);
-		} else {
-			DEBUG ("No enabled account, Showing account dialog");
-			empathy_accounts_dialog_show (GTK_WINDOW (priv->window), NULL);
-		}
 	}
 }
 
@@ -378,7 +379,7 @@ status_icon_key_press_event_cb  (GtkWidget *window,
 	}
 	return FALSE;
 }
-				
+
 static void
 status_icon_activate_cb (GtkStatusIcon     *status_icon,
 			 EmpathyStatusIcon *icon)
@@ -481,7 +482,7 @@ status_icon_create_menu (EmpathyStatusIcon *icon)
 
 static void
 status_icon_connection_changed_cb (EmpathyAccountManager *manager,
-				   McAccount *account,
+				   EmpathyAccount *account,
 				   TpConnectionStatusReason reason,
 				   TpConnectionStatus current,
 				   TpConnectionStatus previous,
